import { Elysia, t } from "elysia";
import mysql from "mysql2/promise";

const pool = mysql.createPool({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

type Member = {
  id: number;
  name: string;
  phone: string;
};

// dummy data
const data = {
  foods: ["Banana", "Apple", "Watermelon"],
};

const app = new Elysia().get("/", () => "Hello Elysia").listen(3000);

app.group("/api", (api) => {
  return api
    .post(
      "/create",
      async ({ body }) => {
        const { name, phone } = body;
        if (!name || !phone) {
          return {
            status: "error",
            message: "Name and phone are required.",
          };
        }

        const [res] = await pool.execute(
          "INSERT INTO `member` (name, phone) VALUES (?, ?)",
          [name, phone]
        );

        return {
          status: "success",
          message: `${name} is added to the list.`,
          data: {
            name,
            phone,
          } as Omit<Member, "id">,
        };
      },
      {
        body: t.Object({
          name: t.String(),
          phone: t.String(),
        }),
      }
    )

    .get("/read", async () => {
      const [rows] = await pool.query<any[]>("SELECT * FROM `member`");

      if (rows.length === 0) {
        return {
          status: "error",
          message: "No members found.",
        };
      }

      return {
        status: "success",
        message: "Members collected.",
        data: rows as Member[],
      };
    })

    .put(
      "/update",
      async ({ body }) => {
        const { id, name, phone } = body;

        const [res] = await pool.execute<mysql.ResultSetHeader>(
          "UPDATE `member` SET name = ?, phone = ? WHERE id = ?",
          [name, phone, id]
        );

        if (res.affectedRows === 0) {
          return {
            status: "error",
            message: `Member with ID ${id} not found.`,
          };
        }

        const [rows] = await pool.query<any[]>(
          "SELECT * FROM `member` WHERE id = ?",
          [id]
        );
        return {
          status: "success",
          message: `Member with ID ${id} is updated.`,
          data: rows[0],
        };
      },
      {
        body: t.Object({
          id: t.Number(),
          name: t.String(),
          phone: t.String(),
        }),
      }
    )

    .delete(
      "/delete",
      async ({ body: { id } }) => {
        const [rows] = await pool.query<any[]>(
          "SELECT * FROM `member` WHERE id = ?",
          [id]
        );
        if (rows.length === 0) {
          return {
            status: "error",
            message: `Member with ID ${id} not found.`,
          };
        }
        await pool.execute("DELETE FROM `member` WHERE id = ?", [id]);
        return {
          status: "success",
          message: `Member with ID ${id} is deleted.`,
          data: rows[0],
        };
      },
      { body: t.Object({ id: t.Number() }) }
    );
});

console.log(
  `🦊 Elysia is running at ${app.server?.hostname}:${app.server?.port}`
);
